        // Implementation of the StopWatch interface.
import java.util.*;

class TimingDevice implements StopWatch {
    public TimingDevice(){
        // Make sure everything is reset to start with.
        reset();
    }

    public void reset(){
        setState(RESET);
        setElapsed(0);
    }

    public void start(){
        if(getState() == RESET){
            setLatestStartTime(new Date());
            setState(RUNNING);
        }
    }

    public void stop(){
        if(getState() == RUNNING){
            // Calculate the elapsed time at this point.
            recordElapsedTime();
            setState(STOPPED);
        }
        else if(getState() == SUSPENDED){
            setState(STOPPED);
        }
        else{
            // Inappropriate action.
        }
    }

    public void suspend(){
        if(getState() == RUNNING){
            recordElapsedTime();
            setState(SUSPENDED);
        }
    }

    public void resume(){
        if(getState() == SUSPENDED){
            // Set the resumption time.
            setLatestStartTime(new Date());
            setState(RUNNING);
        }
    }

    public long getElapsedTime(){
        // We must calculate how much time has passed since the
        // elapsed time was last calculated.
        long increment = 0;
        if(getState() == RUNNING){
            Date now = new Date();
            increment = now.getTime()-getStartTimeInMillis();
        }
        // Set the new value.
        setElapsed(getElapsed() + increment);
        return getElapsed();
    }

    public int getState(){
        return state;
    }

    private void setState(int s){
        state = s;
    }

        // Calculate how much has elapsed to this point.
    private void recordElapsedTime(){
        Date now = new Date();
        // Find out how much time has passed since
        // latestStartTime.
        long increment = now.getTime()-getStartTimeInMillis();
        // Add it in to what has elapsed so far.
        setElapsed(getElapsed()+increment);
    }

    private long getElapsed(){
        return elapsed;
    }

    private void setElapsed(long time){
        elapsed = time;
    }

    private long getStartTimeInMillis(){
        return getLatestStartTime().getTime();
    }

    private Date getLatestStartTime(){
        return latestStartTime;
    }

    private void setLatestStartTime(Date now){
        latestStartTime = now;
    }

    // Current state: RESET, RUNNING, SUSPENDED, STOPPED.
    private int state = RESET;
    // Keep track of the most recent start time and
    // how much time has elapsed in the preceding periods.
    private Date latestStartTime;
    private long elapsed = 0;
}
