    // Illustrate methods of the java.lang.Class class.
class ClassExampleMain {
    public static void main(String[] args){
        // Create some objects whose classes will be examined.
        final String obj = new String();
        final int[] arr = new int[10];

        // Obtain class objects for the String, the array
        // and the primitive int type.
        final Class[] classes = {
            obj.getClass(),
            arr.getClass(),
            int.class,
        };

        // Show details of the example classes.
        final int numClasses = classes.length;
        for(int i = 0; i < numClasses; i++){
            showClassDetails(classes[i]);
        }
    }

        // Show a few details on the given class.
    public static void showClassDetails(Class c){
        // The class's name.
        System.out.println(c.getName());
        if(c.isPrimitive()){
            System.out.println(" Is a primitive type.");
        }
        else if(c.isArray()){
            Class baseType = c.getComponentType();
            System.out.println(" Is an array of: "+baseType.getName());
        }
        else{
            // A class.
            final Class superClass = c.getSuperclass();
            if(superClass != null){
                System.out.println(" Extends: "+superClass.getName());
            }
            if(c.isInterface()){
                System.out.println(" Is an interface.");
            }
            // See if this class implements one or more interfaces.
            final Class[] interfaces = c.getInterfaces();
            for(int i = 0; i < interfaces.length; i++){
                System.out.println(" Implements: "+
                                   interfaces[i].getName());
            }
        }
    }
}
