        // Contact the UDP echo server running on hostname.

import java.net.*;
import java.io.*;

class UDPEchoClient extends UDPUser {
    public UDPEchoClient(String hostname) throws UnknownHostException,
                                                 SocketException {
        super(hostname);
    }

        // Send the given strings to the UDP echo service.
    public void run(String[] sampleStrings){
        // The well-known port of the UDP echo service.
        final int EchoPort = 7;

        try{
            // Get the socket we are using.
            DatagramSocket socket = getSocket();
            // Get the address of the server.
            InetAddress server = getCorrespondent();

            // Create an array of byte[] equivalents of the strings
            // to be sent.
            final int numStrings = sampleStrings.length;
            byte[][] byteStrings = new byte[numStrings][];

            // Keep track of the longest. We need to know this
            // in order to create the packet for the replies.
            int maxBytes = -1;
            for(int i = 0; i < numStrings; i++){
                // Turn the String into an array of bytes.
                byteStrings[i] = sampleStrings[i].getBytes();
                // See if this is the longest so far.
                final int length = byteStrings[i].length;
                if(length > maxBytes){
                    maxBytes = length;
                }
            }

            // Create a datagram packet to send.
            // We use the same packet for each byte string.
            DatagramPacket packet =
                        new DatagramPacket(byteStrings[0],0,server,EchoPort);
            // Create a buffer and packet for receiving the reply.
            byte[] replySpace = new byte[maxBytes];
            DatagramPacket reply =
                new DatagramPacket(replySpace,replySpace.length);

            // Send each byte[] string in turn.
            for(int i = 0; i < byteStrings.length; i++){
                // Setup up the packet for this string.
                packet.setData(byteStrings[i]);
                packet.setLength(byteStrings[i].length);

                // Send the packet.
                System.out.println("Sending: "+sampleStrings[i]);
                // Send the packet (repeatedly if necessary)
                // and receive the reply.
                sendAndReceive(packet,reply);

                // Print out the reply.
                System.out.print("Received: ");
                System.out.write(replySpace,0,reply.getLength());
                System.out.println();
            }
            socket.close();
        }
        catch(IOException e){
            System.err.println("IOException: "+e.getMessage());
        }
    }
}
