class RangeCounts {
    // The default values for a chart.
    public static final int defaultMin = 1, defaultMax = 100,
                                        defaultUnitsPerBar = 10;
    public RangeCounts(){
        this(defaultMin,defaultMax,defaultUnitsPerBar);
    }

    public RangeCounts(int min,int max,int unitsPerBar){
        setMin(min);
        setMax(max);
        setUnitsPerBar(unitsPerBar);
        // Set up counts for the right number of bars.
        final int range = max-min+1;
        int numBars = range/unitsPerBar;
        if((range % unitsPerBar) > 0){
            numBars++;
        }
        setCounts(new int[numBars]);
    }

    public void addValue(int value){
        if((value >= min) && (value <= max)){
            final int barNumber = (value-min)/unitsPerBar;
            getCounts()[barNumber]++;
        }
    }

    public int getMin(){
        return min;
    }

    protected void setMin(int m){
        min = m;
    }

    public int getMax(){
        return max;
    }

    protected void setMax(int m){
        max = m;
    }

    public int getUnitsPerBar(){
        return unitsPerBar;
    }

    public void setUnitsPerBar(int u){
        unitsPerBar = u;
    }

    public int[] getCounts(){
        return counts;
    }

    public int numCounts(){
        final int[] c = getCounts();
        return c != null ? c.length : 0;
    }

    protected void setCounts(int[] c){
        if(c != null){
            counts = c;
        }
    }

    // Limits of the values to be included.
    private int min = 1, max = 100;
    // How many units per par.
    private int unitsPerBar = 10;
    // How many in each bar so far.
    private int[] counts;
}
