class JoinCoordinator implements Runnable {
    public JoinCoordinator() {
        initCounters();
    }

        // Set up an array of threads to run multiple Counters.
    protected void initCounters(){
        // Where the threads are to be stored.
        Thread[] counters = getCounters();

        // Create the counters and threads.
        for(int i = 0; i < counters.length; i++){
            // Specify the range over which the counter operates.
            final int min = 10*i+1, max = min+9;
            counters[i] = new Thread(new FairCounter(min,max));
        }
        System.out.println("All counters created.");
    }

    public void run(){
        // Find out my priority.
        Thread myThread = Thread.currentThread();
        int myPriority = myThread.getPriority();
        Thread[] counters = getCounters();

        for(int i = 0; i < counters.length; i++){
            // Ensure that the threads don't start until I am
            // ready for them to run.
            counters[i].setPriority(myPriority-1);
            counters[i].start();
        }

        System.out.println("All counters ready to run.");
        waitForCountersToFinish(myPriority);
        System.out.println("All the threads have finished.");
    }

    protected void waitForCountersToFinish(int myPriority){
        // Crank down my priority below the other threads in order to
        // let them battle it out with the minimum of interference.
        Thread myThread = Thread.currentThread();
        myThread.setPriority(myPriority-1);

        // Wait for all the counters to finish.
        Thread[] counters = getCounters();
        for(int i = 0; i < counters.length; i++){
            // Anticipate being interrupted before the
            // join is complete.
            boolean interrupted;
            do{
                try{
                    interrupted = false;
                    counters[i].join();
                }
                catch(InterruptedException e){
                    interrupted = true;
                }
            }
            while(interrupted);
        }
    }

    protected Thread[] getCounters(){
        return counters;
    }

    // Create the array to hold the threads.
    private final int numCounters = 3;
    private Thread[] counters = new Thread[numCounters];
}
